@@ -125,14 +125,10 @@ impl Default for Timer {
     }
 }
 
-/// A rules for todo list filtering. Setting a field to None or empty vector
-/// means that the corresponding property is not checked.
-/// All text comparisons are case-insensitive.
+/// Filter rules for special entities: projects, contexts, tags.
 #[derive(Debug, Clone)]
-pub struct Conf {
-    /// Range of todo IDs
-    pub range: ItemRange,
-    /// List of all project tags that a todo must include. The search
+pub struct TagFilter {
+    /// List of all project that a todo must include. The search
     /// supports very limited pattern matching:
     /// * `foo*` - finds all todos with projects that starts with `foo`
     /// * `*foo` - finds all todos with projects that ends with `foo`
@@ -141,7 +137,7 @@ pub struct Conf {
     /// * none - select todos with no contexts
     /// * any - select todos that have at least one context
     pub projects: Vec<String>,
-    /// List of all context tags that a todo must include. The search
+    /// List of all context that a todo must include. The search
     /// supports very limited pattern matching:
     /// * `foo*` - finds all todos with contexts that starts with `foo`
     /// * `*foo` - finds all todos with contexts that ends with `foo`
@@ -159,12 +155,26 @@ pub struct Conf {
     /// * none - select todos with no tags
     /// * any - select todos that have at least one tag
     pub tags: Vec<String>,
+}
+
+/// A rules for todo list filtering. Setting a field to None or empty vector
+/// means that the corresponding property is not checked.
+/// All text comparisons are case-insensitive.
+#[derive(Debug, Clone)]
+pub struct Conf {
+    /// Range of todo IDs
+    pub range: ItemRange,
     /// A text that any of text, project, or context must contain
     pub regex: Option<String>,
     /// If it is `true`, `regex` is treated as regular expression. If `use_regex`
     /// is `false`, the value of `regex` is just a substring to search for
     pub use_regex: bool,
 
+    /// Todos must contain the following values to be included in the list.
+    pub include: TagFilter,
+    // Todos that include the following values are excluded from the list.
+    pub exclude: TagFilter,
+
     /// All incomplete, completed, or both types of todos
     pub all: TodoStatus,
 
@@ -188,9 +198,8 @@ impl Default for Conf {
     fn default() -> Conf {
         Conf {
             range: ItemRange::None,
-            projects: Vec::new(),
-            contexts: Vec::new(),
-            tags: Vec::new(),
+            include: TagFilter { projects: Vec::new(), contexts: Vec::new(), tags: Vec::new() },
+            exclude: TagFilter { projects: Vec::new(), contexts: Vec::new(), tags: Vec::new() },
             regex: None,
             use_regex: false,
 
@@ -268,14 +277,17 @@ fn vec_match(task_list: &[String], filter: &[String]) -> bool {
 }
 
 fn filter_context(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::IDVec {
-    if c.contexts.is_empty() {
+    if c.include.contexts.is_empty() && c.exclude.contexts.is_empty() {
         return v;
     }
 
     let mut new_v: todo::IDVec = Vec::new();
     for i in v.iter() {
         let idx = *i;
-        if vec_match(&tasks[idx].contexts, &c.contexts) {
+        if !c.exclude.contexts.is_empty() && vec_match(&tasks[idx].contexts, &c.exclude.contexts) {
+            continue;
+        }
+        if c.include.contexts.is_empty() || vec_match(&tasks[idx].contexts, &c.include.contexts) {
             new_v.push(idx);
         }
     }
@@ -283,14 +295,17 @@ fn filter_context(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::ID
 }
 
 fn filter_project(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::IDVec {
-    if c.projects.is_empty() {
+    if c.include.projects.is_empty() && c.exclude.projects.is_empty() {
         return v;
     }
 
     let mut new_v: todo::IDVec = Vec::new();
     for i in v.iter() {
         let idx = *i;
-        if vec_match(&tasks[idx].projects, &c.projects) {
+        if !c.exclude.projects.is_empty() && vec_match(&tasks[idx].projects, &c.exclude.projects) {
+            continue;
+        }
+        if c.include.projects.is_empty() || vec_match(&tasks[idx].projects, &c.include.projects) {
             new_v.push(idx);
         }
     }
@@ -298,7 +313,7 @@ fn filter_project(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::ID
 }
 
 fn filter_tag(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::IDVec {
-    if c.tags.is_empty() {
+    if c.include.tags.is_empty() && c.exclude.tags.is_empty() {
         return v;
     }
 
@@ -309,7 +324,10 @@ fn filter_tag(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::IDVec
         for (k, _v) in tasks[idx].tags.iter() {
             tag_list.push(k.to_string());
         }
-        if vec_match(&tag_list, &c.tags) {
+        if !c.exclude.tags.is_empty() && vec_match(&tag_list, &c.exclude.tags) {
+            continue;
+        }
+        if c.include.tags.is_empty() || vec_match(&tag_list, &c.include.tags) {
             new_v.push(idx);
         }
     }
@@ -474,18 +492,11 @@ fn filter_threshold(tasks: &todo::TaskSlice, v: todo::IDVec, c: &Conf) -> todo::
     let flt = if let Some(thr) = &c.thr {
         thr.clone()
     } else {
-        DateRange{
-            days: ValueRange{low: INCLUDE_NONE, high: 0, },
-            span: ValueSpan::Range,
-        }
+        DateRange { days: ValueRange { low: INCLUDE_NONE, high: 0 }, span: ValueSpan::Range }
     };
     let mut new_v: todo::IDVec = Vec::new();
     for i in v.iter() {
         let idx = *i;
-        if tasks[idx].threshold_date.is_none() {
-            new_v.push(idx);
-            continue;
-        }
         if date_in_range(&tasks[idx].threshold_date, &flt) {
             new_v.push(idx);
         }
@@ -519,7 +530,7 @@ fn date_in_range(date: &Option<chrono::NaiveDate>, range: &DateRange) -> bool {
                 let diff = *d - today;
                 diff.num_days() >= range.days.low && diff.num_days() <= range.days.high
             } else {
-                false
+                range.days.low == INCLUDE_NONE || range.days.high == INCLUDE_NONE
             }
         }
         _ => false,
